/*
 * bg_main.c
 * The main entry point for bgrotd
 * bgrot project
 * $Fuller: myprograms/bgrot/server/bg_main.c,v 1.10 2003/03/24 16:28:19 fullermd Exp $
 */

#include <pthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <bgrot.h>
#include <bgrotd.h>

/*
 * Global vars.
 * Note that many of these are going to be accessed cross-thread, so
 * we're going to have to slap some mutexes around them.
 */

/* Server config */
BGROTD_CONF *runcf;
pthread_mutex_t runcf_mtx=PTHREAD_MUTEX_INITIALIZER;

/* The current image */
/* ** because it'll be moving, not just changing contents */
IMGENT **cur_img;
IMGENT **nxt_img; /* Covered by same mutex */
pthread_mutex_t cur_img_mtx=PTHREAD_MUTEX_INITIALIZER;
pthread_cond_t cur_img_cond=PTHREAD_COND_INITIALIZER;

/* Various threads */
pthread_t img_thr;


/* Non-thread-shared globals */
char *progname;
int debug=0;


/* Launch into the vortex */
int
main(int argc, char *argv[])
{
	int optch;
	char *conffile;

	progname = *argv;
	conffile = NULL;

	/* First, parse our command-line args */
	while( (optch = getopt(argc, argv, "dc:")) != -1)
		switch((char)optch)
		{
			case 'c':
				/* Config file */
				conffile = optarg;
				break;
			case 'd':
				/* Debug */
				debug=1;
				break;
			default:
				usage();
				/* NOTREACHED */
		}
	
	/* Make sure we have a conffile! */
	if(conffile==NULL)
	{
		fprintf(stderr, "No config file specified!\n");
		usage(); /* Doesn't return */
	}

	/*
	 * Now parse it out.  No need to mutex the variable since we haven't
	 * started any other threads yet by this point.
	 */
	runcf = gencf(conffile);

	/*
	 * Everything from here on will be part of the 'main loop', so let's
	 * jump on into that now.
	 */
	if(debug==0)
		if( bg_daemon() < 0 )
		{
			fprintf(stderr, "Error daemonizing\n");
			exit(1);
		}
	bg_core();

	/* bg_core() should never return.  If it does, we're in trouble */
	exit(1);
}
